 /*
  * jconsole.cpp - Scrolling Virtual Framebuffer Object With Japanese Support
  * Copyright (c)2004 Pawel Szymczykowski <pawel@makenai.net>
  *
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * It is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with Foobar; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

// Note : There seems to be some sort of overflow when you make the screen too small.
// Not a problem most of the time.. but just be aware of it.

#include "jconsole.h"

JConsole::JConsole(int x1, int y1, int x2, int y2, BitFont *hankaku, BitFont *zenkaku, int linebuffer_size) 
{
	// *ToDo: Do a check on font widths.. make sure hankaku is really _half_
	this->hankaku = hankaku;
	this->zenkaku = zenkaku;
	this->x1 = x1;
	this->y1 = y1;
	this->x2 = x2;
	this->y2 = y2;
	this->cols = this->get_width() / this->hankaku->get_width();
	this->rows = this->get_height() / this->hankaku->get_height();
	this->linebuffer_size = linebuffer_size;
	if (this->rows > this->linebuffer_size) 
		this->rows = this->linebuffer_size; // adjust rows
	this->linebuffer = (char **) gm_malloc(this->linebuffer_size * sizeof(char *)); 
	for(int i=0; i < this->linebuffer_size; i++)
	{
		this->linebuffer[i] = (char *) gm_malloc((this->cols + 1) * sizeof(char));
		this->linebuffer[i][0] = '\0';
	}
	this->current_line = 0;
	this->disp_line = 0;
}

JConsole::~JConsole() 
{
	for(int i=0; i < this->linebuffer_size; i++)
		gm_free(this->linebuffer[i]);
	gm_free(this->linebuffer);
}

void JConsole::clear()
{
	for(int i=0; i < this->linebuffer_size; i++) this->linebuffer[i][0] = '\0';
	this->current_line = 0;
	this->disp_line = 0;
}

void JConsole::backspace()
{
	int length = gm_lstrlen(this->linebuffer[this->current_line]);
	if (length) this->linebuffer[this->current_line][length - 1] = '\0';
	/*
	else 
	{
		if (this->current_line > 0)
		{
			this->current_line--;
			length = gm_lstrlen(this->linebuffer[this->current_line]);
			if (length) this->linebuffer[this->current_line][length - 1] = '\0';
		}
	}
	*/
}

void JConsole::line_feed()
{
	if (this->current_line < this->linebuffer_size - 1)
	{
		this->current_line++; 
	} else {
		// Yeah. This is likely causing a crash.
		// do the pointer shuffle!
		char * tmp = linebuffer[0];
		for (int i=0; i < this->current_line; i++)
		{
			// gm_strcpy(this->linebuffer[i], this->linebuffer[i+1]);
			this->linebuffer[i] = this->linebuffer[i+1];
		}
		this->linebuffer[this->current_line] = tmp;
	}
}

int JConsole::next_word_length(char * string, int pos, int maxlen)
{
	int i;
	for (i=pos;i<maxlen;i++)
	{
		if ((string[i] == ' ') ||  // spaces break a word
		    (string[i] == '\n') || // ..and newlines..
		    ((i+1<=maxlen) && ISEUC(string[i], string[i+1])) // so does Japanese!
		) break;
	}
	return (i - pos);
}

void JConsole::print(char * string)
{
	this->print(string, 0x00);
}
	
void JConsole::print(char * string, int color)
{
	int i=0;
	int length = gm_lstrlen(string);
	while(this->linebuffer[this->current_line][i] != '\0') i++; // append
	for (int j=0;j<length;j++)
	{
		if (string[j] == ' ' || j == 0) 
		{ // Advanced word wrapping (let it be turned on or off)
			if (this->next_word_length(string, j+1, length) >= (this->cols - i))
			{
				this->linebuffer[this->current_line][i] = '\0'; 
				this->line_feed(); 
				i=0; 
				if (j != 0 ) continue; // Clean me up!
			}
		}
		if (string[j] == '\n') 
		{ // treat newlines
			this->linebuffer[this->current_line][i] = '\0'; 
			this->line_feed(); 
			i=0; 
			continue;
		}
		if ((i>=cols) || (i+1>=cols) && (j+1<=length) && ISEUC(string[j],string[j+1])) 
		{ // catch everything else including Japanese
			this->linebuffer[this->current_line][i] = '\0';
			this->line_feed();
			i=0;
		}
		this->linebuffer[this->current_line][i++] = string[j];
	}
	this->linebuffer[this->current_line][i] = '\0'; // finish up with a terminator
}

void JConsole::draw(GPDRAWSURFACE * surface)
{ 
	for (int y=0; y < this->rows; y++)
	{
		int line_length = gm_lstrlen(linebuffer[this->disp_line + y]);
		for (int x=0;x<line_length;x++)
			if ((x+1<=line_length) && (ISEUC(this->linebuffer[this->disp_line + y][x], this->linebuffer[this->disp_line + y][x+1])))
			{
				this->zenkaku->putchar(surface, 
					EUC2IDX(this->linebuffer[this->disp_line + y][x], this->linebuffer[this->disp_line + y][x+1]), 
					this->x1 + (x*this->hankaku->get_width()), 
					this->y1 + (y*this->hankaku->get_height()), 
					0x00);
				x++;
			} else { 
				this->hankaku->putchar(surface, 
					this->linebuffer[this->disp_line + y][x] - 1, 
					this->x1 + (x*this->hankaku->get_width()), 
					this->y1 + (y*this->hankaku->get_height()), 
					0x00);
			}
	}
}

int JConsole::scroll_up()
{
	if (this->disp_line > 0)
	{
		this->disp_line--;
		return(0);
	} else return(1);
}

int JConsole::scroll_dn()
{
	if ((this->disp_line + this->rows) < this->current_line)
	{
		this->disp_line++;
		return(0);
	} else return(1);
}

void JConsole::home()
{
	this->disp_line = 0;
}

void JConsole::end()
{
	if (this->current_line > this->rows)
		this->disp_line = this->current_line - this->rows;
}

int JConsole::get_cur_line_length()
{
	int i=0;
	while(this->linebuffer[this->current_line][i] != '\0') i++;
	return i;
}

int JConsole::get_width()
{ return(this->x2 - this->x1); }

int JConsole::get_height()
{ return(this->y2 - this->y1); }

int JConsole::get_num_rows() 
{ return this->rows; }

int JConsole::get_num_cols() 
{ return this->cols; }

int JConsole::get_num_lines() 
{ return this->current_line; }

int JConsole::get_top_line() 
{ return this->disp_line; }
