 /*
  * jconsole.h - Scrolling Virtual Framebuffer Object With Japanese Support
  * Copyright (c)2004 Pawel Szymczykowski <pawel@makenai.net>
  *
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * It is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with Foobar; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#ifndef __JCONSOLE_H__
#define __JCONSOLE_H__

#include "gpmem.h"
#include "bitfont.h"

#define ISEUCBYTE(A)	(A >= 161 && A <= 254)
#define ISEUC(A,B)     	(ISEUCBYTE(A) && ISEUCBYTE(B))
#define EUC2IDX(A,B)	(((A - 161) * 94) + (B - 161)) // Secret Formula: (KU - 1 * 94) + (TEN - 1)

class JConsole
{
	public:
		JConsole(int x1, int y1, int x2, int y2, 
			BitFont *hankaku, BitFont *zenkaku, int linebuffer_size);
		virtual ~JConsole();
		void print(char * string, int color);
		void print(char * string);
		void draw(GPDRAWSURFACE *surface);
		int get_num_rows();
		int get_num_cols();
		int get_num_lines();
		int get_top_line();
		int get_cur_line_length();
		int get_width();
		int get_height();
		int scroll_up();
		int scroll_dn();
		void home();
		void end();
		void backspace();
		void clear();
	private:
		void line_feed();
		int next_word_length(char * string, int pos, int maxlen);
		int x1, y1, x2, y2;
		BitFont *hankaku;
		BitFont *zenkaku;
		int linebuffer_size;
		int disp_line;
		char ** linebuffer;
		int rows;
		int cols;
		int current_line;
};

#endif /*__JCONSOLE_H__*/
