 /*
  * keyboard.h - Virtual Keyboard Matrix Definitions
  * Copyright (c)2004 Pawel Szymczykowski <pawel@makenai.net>
  *
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * It is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with Foobar; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#ifndef __KEYBOARD_H__
#define __KEYBOARD_H__

#include "img/keyboard.xbm"

#define KEYBOARD_ROWS 4
#define KEYBOARD_COLS 11

// Special Keys
#define WAEI_MODE 129
#define EIWA_MODE 130
#define LOOKUP    131

typedef struct
{
	unsigned int xoff, yoff, width, height;
	unsigned int up, down, left, right;
	char value;
} keydata;

static keydata keymap[] = 
{ 
	// Row 1
	{9,9,23,23,	29,11,10,1,	'q'}, 		//  0
	{33,9,23,23,	29,12,0,2,	'w'}, 		//  1
	{57,9,23,23,	29,13,1,3,	'e'}, 		//  2
	{81,9,23,23,	29,14,2,4,	'r'}, 		//  3
	{105,9,23,23,	29,15,3,5,	't'}, 		//  4
	{129,9,23,23,	29,16,4,6,	'y'}, 		//  5
	{153,9,23,23,	29,17,5,7,	'u'}, 		//  6
	{177,9,23,23,	29,18,6,8,	'i'}, 		//  7
	{201,9,23,23,	29,19,7,9,	'o'}, 		//  8
	{225,9,23,23,	29,19,8,10,	'p'}, 		//  9
	{257,9,35,23,	30,20,9,0,	WAEI_MODE},	// 10
	// Row 2
	{9,33,23,23,	0,21,20,12,	'a'}, 		// 11
	{33,33,23,23,	1,22,11,13,	's'}, 		// 12
	{57,33,23,23,	2,23,12,14,	'd'}, 		// 13
	{81,33,23,23,	3,24,13,15,	'f'}, 		// 14
	{105,33,23,23,	4,25,14,16,	'g'}, 		// 15
	{129,33,23,23,	5,26,15,17,	'h'}, 		// 16
	{153,33,23,23,	6,27,16,18,	'j'}, 		// 17
	{177,33,23,23,	7,28,17,19,	'k'}, 		// 18
	{201,33,23,23,	8,28,18,20,	'l'}, 		// 19
	{257,33,35,23,	10,30,19,11,	EIWA_MODE},	// 20
	// Row 3
	{9,57,23,23,	11,29,28,22,	'z'},		// 21
	{33,57,23,23,	12,29,21,23,	'x'},		// 22
	{57,57,23,23,	13,29,22,24,	'c'}, 		// 23
	{81,57,23,23,	14,29,23,25,	'v'},		// 24
	{105,57,23,23,	15,29,24,26,	'b'},		// 25
	{129,57,23,23,	16,29,25,27,	'n'}, 		// 26
	{153,57,23,23,	17,29,26,28,	'm'},		// 27
	{177,57,23,23,	18,29,27,21,	'-'},		// 28
	// Row 4
	{33,81,143,23,	24,3,30,30,	' '},		// 29
	{243,81,49,23,	20,10,29,29,	LOOKUP}		// 30
};

#endif /*__KEYBOARD_H__*/
